@@ -647,6 +647,7 @@ def process_damask_orientatons(ori_data):
         'type': 'quat',
         'quaternions': ori_data,
         'unit_cell_alignment': {'x': 'a'},
+        'quat_component_ordering': 'scalar-vector',
         'P': -1,
     }
 
@@ -867,6 +868,7 @@ def validate_orientations(orientations):
                 radians.
             unit_cell_alignment : dict
                 Alignment of the unit cell.
+            quat_component_ordering: str ("scalar-vector" or "vector-scalar")
 
     Returns
     -------
@@ -884,6 +886,8 @@ def validate_orientations(orientations):
                 The "P" constant, either +1 or -1, as defined within [1]. If included in
                 the original `orientations` dict, this value will be unchanged. Otherwise,
                 this will be set to +1.
+            quat_component_ordering: str ("scalar-vector" or "vector-scalar")
+                Value will be set to "scalar-vector".
 
     References
     ----------
@@ -899,8 +903,11 @@ def validate_orientations(orientations):
     eulers = orientations.get('euler_angles')
     euler_is_degs = orientations.get('euler_degrees')
     quats = orientations.get('quaternions')
+    quats_comp_order = orientations.get('quat_component_ordering')
     alignment = orientations.get('unit_cell_alignment')
 
+    ALLOWED_QUAT_ORDER = ['scalar-vector', 'vector-scalar']
+
     P = orientations.get('P', 1)
     if P not in [-1, 1]:
         raise ValueError('If specified, `P` should be +1 or -1.')
@@ -937,6 +944,7 @@ def validate_orientations(orientations):
 
         # Convert Euler angles to quaternions:
         quaternions = euler2quat(euler_angles, degrees=euler_is_degs, P=P)
+        quats_comp_order = 'scalar-vector'
 
     elif ori_type == 'quat':
         if quats is None:
@@ -954,6 +962,17 @@ def validate_orientations(orientations):
                    f'array of shape (R, 4), but shape passed was: {quaternions.shape}.')
             raise ValueError(msg)
 
+        if quats_comp_order == 'vector-scalar':
+            # "normalise" to scalar-vector convention:
+            quaternions = np.roll(quaternions, 1, axis=1)
+            quats_comp_order = 'scalar-vector'
+        
+    if quats_comp_order not in ALLOWED_QUAT_ORDER:
+        msg = (f'Quaternion component order key `quat_component_ordering` must be '
+                f'specified as either "scalar-vector" or "vector-scalar". Actual '
+                f'value was: "{quats_comp_order}".')
+        raise ValueError(msg)
+        
     # To ensure maximum precision of quaternions, cast to longdouble (although note that
     # precision is system-dependent):
     quaternions = quaternions.astype(np.longdouble)
@@ -974,6 +993,7 @@ def validate_orientations(orientations):
     orientations_valid = {
         'type': 'quat',
         'quaternions': quaternions,
+        'quat_component_ordering': quats_comp_order,
         'unit_cell_alignment': alignment,
         'use_max_precision': orientations.get('use_max_precision', False),
         'P': P,
@@ -1370,7 +1390,7 @@ def get_constituent_material_idx(material_constituent_idx):
 
 def get_volume_element_materials(volume_element, homog_schemes=None, phases=None, P=-1):
     """Get the materials list from a volume element that can be used to populate
-    the "microstructures" list in a DAMASK materials.yaml file.
+    the "material" list in a DAMASK materials.yaml file.
 
     Parameters
     ----------
@@ -1407,6 +1427,14 @@ def get_volume_element_materials(volume_element, homog_schemes=None, phases=None
     mat_const_idx = get_material_constituent_idx(const_mat_idx)
 
     all_quats = volume_element['orientations']['quaternions']
+    
+    quat_comp_order = volume_element['orientations'].get('quat_component_ordering')
+    if quat_comp_order != 'scalar-vector':
+        msg = (f'Quaternion component ordering (`quat_component_ordering`) should be '
+               f'"scalar-vector", as adopted by DAMASK, but it is actually '
+               f'"{quat_comp_order}"')
+        raise RuntimeError(msg)
+
     const_mat_frac = volume_element['constituent_material_fraction']
     const_ori_idx = volume_element['constituent_orientation_idx']
     const_phase_lab = volume_element['constituent_phase_label']
@@ -1428,8 +1456,9 @@ def get_volume_element_materials(volume_element, homog_schemes=None, phases=None
                 if volume_element['orientations']['unit_cell_alignment'].get('y') == 'b':
                     # Convert from y//b to x//a:
                     hex_transform_quat = axang2quat(
-                        volume_element['orientations']['P'] * np.array([0, 0, 1]),
-                        np.pi/6
+                        volume_element['orientations']['P'] * np.array(
+                            [0, 0, 1], dtype=np.longdouble),
+                        np.longdouble(np.pi/6)
                     )
                     mat_i_const_j_ori = multiply_quaternions(
                         q1=hex_transform_quat,
@@ -1525,3 +1554,19 @@ def prepare_material_yaml_data(mat_dat):
             const['O'] = ori_list
 
     return mat_dat_fmt
+
+def get_coordinate_grid(size, grid_size):
+    """Get the coordinates of the element centres of a uniform grid."""
+
+    grid_size = np.array(grid_size)
+    size = np.array(size)
+
+    grid = np.meshgrid(*[np.arange(i) for i in grid_size])
+    grid = np.moveaxis(np.array(grid), 0, -1)  # shape (*grid_size, dimension)
+
+    element_size = (size / grid_size).reshape(1, 1, -1)
+
+    coords = grid * size.reshape(1, 1, -1) / grid_size.reshape(1, 1, -1)
+    coords += element_size / 2
+
+    return coords, element_size
